
/*******************************************************************************
*	File:		units.h
*	Project:	MCRP09_RX200
*	Version:	2.0
*	Date:		19-11-2013 (DD-MM-YYYY)
*	Author:		Andrea Ghirardello
*	Contents:	No code, only comment file
********************************************************************************

INTERNAL REPRESENTATION OF PHYSICAL VARIABLES


Angles: the interval [0, 2pi) is represented with the interval [0, 65536), with the resolution of
2pi/65536 rad.

	Angle[int]=KA*Angle[rad]
	KA=32768/pi(=10430.37835)

Note that in this way the angle can be considered unsigned in the range [0, 65536), or signed in
the range [-32768, 32768), with identical results. In every case the representation requires a
16bit word.


Trigonometric functions: sin(a), cos(a) are normalized to NORMVAL=16384.

	Internal_sin(a[int])=NORMVAL*sin(a[rad]), NORMVAL=16384
	-NORMVAL<=Internal_sin()<=NORMVAL (the same for Internal_cos())


Time: the time is expressed as a multiple of the sampling period Ts.

	Time[int]=KT*Time[sec]
	KT=Fs (Fs=sampling_frequency=1/Ts)


Angular velocity: it is expressed as a function of angles and time, in order to obtain the angular
velocity as the subtractions of two angles in two sampling moments; for resolution reasons, an
amplification is needed, and we choose this amplification equal to NORMVAL=16384.

	Omega[in1]=KO1*Omega[r/s]=(KO1*KT/KA)*Angle[int]/Time[int]

Since we want:

	Omega[in1]=NORMVAL*Angle[int]/Time[int]
	( -> Omega[in1]=NORMVAL*(Angle(n)-Angle(n-k))/k )

We obtain:

	KO1=NORMVAL*KA/KT=NORMVAL*65536/(2*pi*Fs)

The entire speed range cannot, in general, be represented in a 16bit word, but a long is needed.
This high resolution can be useful for some particular calculations, while when the
speed is used to calculate voltages, lesser resolution is enough. To reduce the overall
calculation time, the most effective choice is to have a second representation on the angular
speed, coherent with the voltages and currents representations, which are "per unit" based.
So the second representation of the angular speed is based on a normalized value:

	BASE_SPEED_R_S=MAX_OMEGA_R_S

The so called MAX_OMEGA_R_S is the maximum speed required by the application (NORMVAL=16384).
This value is linked to the maximum frequency (MAX_OMEGA_R_S=2*pi*MAX_FRE_HZ).
The second representation is the following:

	Omega[in2]=KO2*Omega[r/s]
	KO2=NORMVAL/BASE_SPEED_R_S

To pass from a representation to the other we have the following relationship:

	Omega[in2]/KO2=Omega[r/s]=Omega[in1]/KO1 -> Omega[in2]=(KO2/KO1)Omega[in1]
	KO2/KO1=(NORMVAL/MAX_SPEED_R_S)/(NORMVAL*65536/(2*pi*Fs))=(2*pi*Fs)/(65536*MAX_OMEGA_R_S)
	KO1/KO2=65536*MAX_FRE_HZ/Fs


Voltage: we can start our considerations from the maximum voltage readable by the A/D converter;
this value is the maximum DC bus voltage and it is related to the maximum peak phase voltage by
the relation: Vout_pk=(2/3)VB in case of over-modulation; this would already leave a good margin
in voltage representation, but in case of deep flux weakening, the intermediate calculations can
lead to higher voltage values, so we choose as the base voltage value the following:

	BASE_VOLTAGE_VOLT=(2^K)*MAX_VOLTAGE_VOLT, with K related with the application

MAX_VOLTAGE_VOLT is the maximum voltage readable by the A/D converter.
With normal applications, K=1 leaves a margin for the maximum phase voltage equal to 3 times VB,
which is more than enough. The voltage representation becomes:
	
	Voltage[int]=KV*Voltage[Vol]
	KV=NORMVAL/BASE_VOLTAGE_VOLT


Current: the maximum current readable by the A/D converter is chosen as the base value:

	BASE_CURRENT_AMP=MAX_CURRENT_AMP

It is represented with the normalized value NORMVAL=16384:

	Current[int]=KI*Current[Amp]
	KI=NORMVAL/BASE_CURRENT_AMP


Impedance: the base impedance value could be deduced by the base voltage and current values;
in fact the extended value chosen as the base voltage keeps into account the flux weakening,
and no other trick seems to be required. This is true in case of PM motor, but in case of
induction motor, the current can be much higher than the ratio between voltage and the impedance
due to the magnetizing inductance (the most important one), since there is a parallel path in the
equivalent circuit, which doesn't include this inductance. In this case, a higher base impedance
has to be chosen, to have the real inductance represented with a word.

	BASE_IMPEDANCE_OHM=(2^H)*BASE_VOLTAGE_VOLT/BASE_CURRENT_AMP, with H related with the motor

In case of a PM motor, H can be chosen equal to 0, while with an induction motor, it has to be
chosen equal to 2 or 3 (depending mainly on the leakage coefficient sigma).
The internal representation is:

	Impedance[int]=KZ*Impedance[Ohm]
	KZ=NORMVAL/BASE_IMPEDANCE_OHM
	  =NORMVAL*BASE_CURRENT_AMP/((2^H)*BASE_VOLTAGE_VOLT)

We can notice that doing these choices, the calculations become, for example:

	Voltage[int]=Impedance[int]*Current[int]/KZI_V,
	KZI_V=KZ*KI/KV=
		 =(NORMVAL/BASE_IMPEDANCE_OHM)*(NORMVAL/BASE_CURRENT_AMP)/(NORMVAL/BASE_VOLTAGE_VOLT)
		 =NORMVAL*(BASE_VOLTAGE_VOLT/BASE_CURRENT_AMP)/BASE_IMPEDANCE_OHM
		 =NORMVAL/(2^H)


Resistance: the resistance is expressed in function of the "base" resistance, which is kept
equal to the base impedance; this leads usually in a "poor" representation of the resistance
in terms of resolution, but the resistance itself is highly variable with many factors, and
an higher resolution is usually not required.

	BASE_RESISTANCE_OHM=BASE_IMPEDANCE_OHM
	Resistance[int]=KR*Resistance[Ohm]
	KR=NORMVAL/BASE_RESISTANCE_OHM
	  =NORMVAL/BASE_IMPEDANCE_OHM
	  =NORMVAL*BASE_CURRENT_AMP/((2^H)*BASE_VOLTAGE_VOLT)

Inductance: the base inductance value is derived from the impedance and the angular velocity:

	BASE_INDUCTANCE_HEN=BASE_IMPEDANCE_OHM/BASE_SPEED_R_S

so the internal representation become:

	Inductance[int]=KL*Inductance[Henry]
	KL=NORMVAL/BASE_INDUCTANCE_HEN
	  =NORMVAL*BASE_SPEED_R_S/BASE_IMPEDANCE_OHM
	  =NORMVAL*BASE_SPEED_R_S*BASE_CURRENT_AMP/((2^H)*BASE_VOLTAGE_VOLT)


Flux: in a similar way, the "base" flux can be chosen equal to:

	BASE_FLUX_WEB=BASE_VOLTAGE_VOL/BASE_SPEED_R_S

Then we can express the flux as:

	Flux[int]=KF*Flux[volt*sec/rad]
	KF=NORMVAL/BASE_FLUX_WEB


Some calculation relations:

	Impedance[Ohm]=Inductance[Hen]*Omega[r/s]
	Impedance[int]=(KZ/(KL*KO2))*Inductance[int]*Omega[in2]
	(KZ/KL)=1/BASE_SPEED_R_S,
	(1/KO2)=BASE_SPEED_R_S/NORMVAL,
	(KZ/(KL*KO2))=1/NORMVAL, so:
	Impedance[int]=(Inductance[int]*Omega[in2]) >> SH_NORMVAL

	Flux[Web]=Inductance[Hen]*Current[Amp]
	Flux[int]=(KF/(KL*KI))*Inductance[int]*Current[int]
	(KF/(KL*KI))=(2^H)/NORMVAL, so:
	Flux[int]=(Inductance[int]*Current[int]) >> SH_IMPEDANCE
	SH_IMPEDANCE=(SH_NORMVAL-H)

	Voltage[Vol]=Impedance[Ohm]*Current[Amp]
	Voltage[int]=(KV/(KZ*KI))*Impedance[int]*Current[int]
	(KV/(KZ*KI))=(2^H)/NORMVAL, so:
	Voltage[int]=(Impedance[int]*Current[int]) >> SH_IMPEDANCE
	SH_IMPEDANCE=(SH_NORMVAL-H)

	Voltage[Vol]=Flux[Web]*Omega[r/s]
	Voltage[int]=(KV/(KF*KO2)*Flux[int]*Omega[in2]
	(KV/KF)=1/BASE_SPEED_R_S,
	(1/KO2)=BASE_SPEED_R_S/NORMVAL,
	(KV/(KF*KO2)=1/NORMVAL, so:
	Voltage[int]=(Flux[int]*Omega[in2]) >> SH_NORMVAL


IMPORTANT NOTE:
When integrating the voltage to obtain flux, internal representation must be kept
into account:
	Flux[Volt*sec/rad]=SUM(Ts[sec]*V[Volt]) ->
	Flux[int]/KF=SUM(TS[sec]*V[int]/KV)		->
	Flux[int]=SUM(KX*V[int]), KX=(KF/KV)/Fs[Hz]=(KF/KV)/KT

********************************************************************************
*	File end
*******************************************************************************/
